static const int g_max_blur_radius = 6;

RWTexture2D<float4> g_coc_near : register(u0);

#define N 256
groupshared float g_cache[N + 2*g_max_blur_radius];

[numthreads(1, N, 1)]
void main(int3 group_thread_id : SV_GroupThreadID, int3 dispatch_thread_id : SV_DispatchThreadID)
{
  float2 dim;
  g_coc_near.GetDimensions(dim.x, dim.y); 
  
  if (group_thread_id.y < g_max_blur_radius)
  {
    int y = max(dispatch_thread_id.y - g_max_blur_radius, 0);
    g_cache[group_thread_id.y] = g_coc_near[int2(dispatch_thread_id.x,y)].r;
  }
  if (group_thread_id.y >= N-g_max_blur_radius)
  {
    int y = min(dispatch_thread_id.y + g_max_blur_radius, dim.y - 1);
    g_cache[group_thread_id.y+2*g_max_blur_radius] = g_coc_near[int2(dispatch_thread_id.x, y)].r;
  }
  
  g_cache[group_thread_id.y+g_max_blur_radius] = g_coc_near[min(dispatch_thread_id.xy, dim.xy-1)].r;
  
  GroupMemoryBarrierWithGroupSync();
  
  float c = 0.0f;
  for (int i = -g_max_blur_radius; i <= g_max_blur_radius; ++i)
  {
    int k = group_thread_id.y + g_max_blur_radius + i;
    c = max(g_cache[k].r, c);
  }

  g_coc_near[dispatch_thread_id.xy].r = c;
}